@@ -1,7 +1,7 @@
 ##
 ## This file is part of the libsigrokdecode project.
 ##
-## Copyright (C) 2013 Uwe Hermann <uwe@hermann-uwe.de>
+## Copyright (C) 2013-2016 Uwe Hermann <uwe@hermann-uwe.de>
 ##
 ## This program is free software; you can redistribute it and/or modify
 ## it under the terms of the GNU General Public License as published by
@@ -14,8 +14,7 @@
 ## GNU General Public License for more details.
 ##
 ## You should have received a copy of the GNU General Public License
-## along with this program; if not, write to the Free Software
-## Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
+## along with this program; if not, see <http://www.gnu.org/licenses/>.
 ##
 
 import sigrokdecode as srd
@@ -24,7 +23,7 @@ class SamplerateError(Exception):
     pass
 
 class Decoder(srd.Decoder):
-    api_version = 2
+    api_version = 3
     id = 'guess_bitrate'
     name = 'Guess bitrate'
     longname = 'Guess bitrate/baudrate'
@@ -42,8 +41,7 @@ class Decoder(srd.Decoder):
     def putx(self, data):
         self.put(self.ss_edge, self.samplenum, self.out_ann, data)
 
-    def __init__(self, **kwargs):
-        self.olddata = None
+    def __init__(self):
         self.ss_edge = None
         self.first_transition = True
         self.bitwidth = None
@@ -55,21 +53,13 @@ class Decoder(srd.Decoder):
         if key == srd.SRD_CONF_SAMPLERATE:
             self.samplerate = value
 
-    def decode(self, ss, es, data):
+    def decode(self):
         if not self.samplerate:
             raise SamplerateError('Cannot decode without samplerate.')
-        for (self.samplenum, pins) in data:
 
-            data = pins[0]
-
-            # Wait for any transition on the data line.
-            if data == self.olddata:
-                continue
-
-            # Initialize first self.olddata with the first sample value.
-            if self.olddata is None:
-                self.olddata = data
-                continue
+        while True:
+            # Wait for any transition/edge on the data line.
+            self.wait({0: 'e'})
 
             # Get the smallest distance between two transitions
             # and use that to calculate the bitrate/baudrate.
@@ -83,5 +73,3 @@ class Decoder(srd.Decoder):
                     bitrate = int(float(self.samplerate) / float(b))
                     self.putx([0, ['%d' % bitrate]])
                 self.ss_edge = self.samplenum
-
-            self.olddata = data
