@@ -15,14 +15,12 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifndef LIBSIGROKDECODE_SIGROKDECODE_H
-#define LIBSIGROKDECODE_SIGROKDECODE_H
+#ifndef LIBSIGROKDECODE_LIBSIGROKDECODE_H
+#define LIBSIGROKDECODE_LIBSIGROKDECODE_H
 
-#include <Python.h> /* First, so we avoid a _POSIX_C_SOURCE warning. */
 #include <stdint.h>
 #include <glib.h>
 
@@ -156,6 +154,12 @@ struct srd_decoder {
 	 */
 	char *license;
 
+	/** List of possible decoder input IDs. */
+	GSList *inputs;
+
+	/** List of possible decoder output IDs. */
+	GSList *outputs;
+
 	/** List of channels required by this decoder. */
 	GSList *channels;
 
@@ -180,14 +184,20 @@ struct srd_decoder {
 	 */
 	GSList *binary;
 
-	/** List of decoder options.  */
+	/** List of decoder options. */
 	GSList *options;
 
 	/** Python module. */
-	PyObject *py_mod;
+	void *py_mod;
 
 	/** sigrokdecode.Decoder class. */
-	PyObject *py_dec;
+	void *py_dec;
+};
+
+enum srd_initial_pin {
+	SRD_INITIAL_PIN_LOW,
+	SRD_INITIAL_PIN_HIGH,
+	SRD_INITIAL_PIN_SAME_AS_SAMPLE0,
 };
 
 /**
@@ -209,7 +219,7 @@ struct srd_decoder_option {
 	char *id;
 	char *desc;
 	GVariant *def;
-    GSList *values;
+	GSList *values;
 };
 
 struct srd_decoder_annotation_row {
@@ -221,7 +231,7 @@ struct srd_decoder_annotation_row {
 struct srd_decoder_inst {
 	struct srd_decoder *decoder;
 	struct srd_session *sess;
-	PyObject *py_inst;
+	void *py_inst;
 	char *inst_id;
 	GSList *pd_output;
 	int dec_num_channels;
@@ -229,6 +239,46 @@ struct srd_decoder_inst {
 	int data_unitsize;
 	uint8_t *channel_samples;
 	GSList *next_di;
+
+	/** List of conditions a PD wants to wait for. */
+	GSList *condition_list;
+
+	/** Array of booleans denoting which conditions matched. */
+	GArray *match_array;
+
+	/** Absolute start sample number. */
+	uint64_t abs_start_samplenum;
+
+	/** Absolute end sample number. */
+	uint64_t abs_end_samplenum;
+
+	/** Pointer to the buffer/chunk of input samples. */
+	const uint8_t *inbuf;
+
+	/** Length (in bytes) of the input sample buffer. */
+	uint64_t inbuflen;
+
+	/** Absolute current samplenumber. */
+	uint64_t abs_cur_samplenum;
+
+	/** Array of "old" (previous sample) pin values. */
+	GArray *old_pins_array;
+
+	/** Handle for this PD stack's worker thread. */
+	GThread *thread_handle;
+
+	/** Indicates whether new samples are available for processing. */
+	gboolean got_new_samples;
+
+	/** Indicates whether the worker thread has handled all samples. */
+	gboolean handled_all_samples;
+
+	/** Requests termination of wait() and decode(). */
+	gboolean want_wait_terminate;
+
+	GCond got_new_samples_cond;
+	GCond handled_all_samples_cond;
+	GMutex data_mutex;
 };
 
 struct srd_pd_output {
@@ -249,7 +299,7 @@ struct srd_proto_data {
 	void *data;
 };
 struct srd_proto_data_annotation {
-	int ann_format;
+	int ann_class;
 	char **ann_text;
 };
 struct srd_proto_data_binary {
@@ -267,19 +317,6 @@ struct srd_pd_callback {
 	void *cb_data;
 };
 
-/* Custom Python types: */
-
-typedef struct {
-	PyObject_HEAD
-	struct srd_decoder_inst *di;
-	uint64_t start_samplenum;
-	unsigned int itercnt;
-	uint8_t *inbuf;
-	uint64_t inbuflen;
-	PyObject *sample;
-} srd_logic;
-
-
 /* srd.c */
 SRD_API int srd_init(const char *path);
 SRD_API int srd_exit(void);
@@ -290,8 +327,8 @@ SRD_API int srd_session_start(struct srd_session *sess);
 SRD_API int srd_session_metadata_set(struct srd_session *sess, int key,
 		GVariant *data);
 SRD_API int srd_session_send(struct srd_session *sess,
-		uint64_t start_samplenum, uint64_t end_samplenum,
-		const uint8_t *inbuf, uint64_t inbuflen);
+		uint64_t abs_start_samplenum, uint64_t abs_end_samplenum,
+		const uint8_t *inbuf, uint64_t inbuflen, uint64_t unitsize);
 SRD_API int srd_session_destroy(struct srd_session *sess);
 SRD_API int srd_pd_output_callback_add(struct srd_session *sess,
 		int output_type, srd_pd_output_callback cb, void *cb_data);
@@ -309,13 +346,15 @@ SRD_API int srd_decoder_unload_all(void);
 SRD_API int srd_inst_option_set(struct srd_decoder_inst *di,
 		GHashTable *options);
 SRD_API int srd_inst_channel_set_all(struct srd_decoder_inst *di,
-		GHashTable *channels, int unit_size);
+		GHashTable *channels);
 SRD_API struct srd_decoder_inst *srd_inst_new(struct srd_session *sess,
 		const char *id, GHashTable *options);
 SRD_API int srd_inst_stack(struct srd_session *sess,
 		struct srd_decoder_inst *di_from, struct srd_decoder_inst *di_to);
 SRD_API struct srd_decoder_inst *srd_inst_find_by_id(struct srd_session *sess,
 		const char *inst_id);
+SRD_API int srd_inst_initial_pins_set_all(struct srd_decoder_inst *di,
+		GArray *initial_pins);
 
 /* log.c */
 typedef int (*srd_log_callback)(void *cb_data, int loglevel,
@@ -324,8 +363,6 @@ SRD_API int srd_log_loglevel_set(int loglevel);
 SRD_API int srd_log_loglevel_get(void);
 SRD_API int srd_log_callback_set(srd_log_callback cb, void *cb_data);
 SRD_API int srd_log_callback_set_default(void);
-SRD_API int srd_log_logdomain_set(const char *logdomain);
-SRD_API char *srd_log_logdomain_get(void);
 
 /* error.c */
 SRD_API const char *srd_strerror(int error_code);
@@ -340,6 +377,8 @@ SRD_API int srd_lib_version_current_get(void);
 SRD_API int srd_lib_version_revision_get(void);
 SRD_API int srd_lib_version_age_get(void);
 SRD_API const char *srd_lib_version_string_get(void);
+SRD_API GSList *srd_buildinfo_libs_get(void);
+SRD_API char *srd_buildinfo_host_get(void);
 
 #include "version.h"
 
